package com.hwrj.cloud.portal.service;

import com.hwrj.cloud.common.api.CommonPage;
import com.hwrj.cloud.portal.domain.ConfirmOrderResult;
import com.hwrj.cloud.portal.domain.OmsOrderDetail;
import com.hwrj.cloud.portal.domain.OrderParam;
import com.hwrj.cloud.portal.from.OrderFrom;
import com.hwrj.cloud.portal.model.OmsOrder;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * 前台订单管理Service
 * Created by macro on 2018/8/30.
 */
public interface OmsPortalOrderService {


    public void cancelOrderByOne(Long orderId);
    /**
     * 根据用户购物车信息生成确认单信息
     * @param cartIds
     */
    ConfirmOrderResult generateConfirmOrder(List<Long> cartIds);

    /**
     * 根据提交信息生成订单
     */
    @Transactional
    Map<String, Object> generateOrder(OrderParam orderParam);

    /**
     * 支付成功后的回调
     */
    @Transactional
    Integer paySuccess(Long orderId, Integer payType);

    /**
     * 自动取消超时订单
     */
    @Transactional
    Integer cancelTimeOutOrder();

    /**
     * 取消单个超时订单
     */
    @Transactional
    void cancelOrder(Long orderId);

    /**
     * 发送延迟消息取消订单
     */
    void sendDelayMessageCancelOrder(Long orderId);

    /**
     * 确认收货
     */
    void confirmReceiveOrder(Long orderId);

    /**
     * 分页获取用户订单
     */
    CommonPage<OmsOrderDetail> list(Integer status, Integer pageNum, Integer pageSize);

    /**
     * 根据订单ID获取订单详情
     */
    OmsOrderDetail detail(Long orderId);

    /**
     * 用户根据订单ID删除订单
     */
    void deleteOrder(Long orderId);

    /**
     * 新建订单
     * @param orderFrom
     */
    Map<String, Object> save(OrderFrom orderFrom);

    Map<String, Object> saveOrder(OrderParam orderParam,OrderFrom orderFrom);


    OmsOrder selectById(Long orderId);

    String getProductName(Long orderId);
}
