package com.hwrj.cloud.portal.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.hwrj.cloud.common.exception.Asserts;
import com.hwrj.cloud.portal.dao.UmsMemberDao;
import com.hwrj.cloud.portal.domain.UmsMemberPlaformDo;
import com.hwrj.cloud.portal.mapper.*;
import com.hwrj.cloud.portal.model.*;
import com.hwrj.cloud.portal.domain.MemberDetails;
import com.hwrj.cloud.portal.service.UmsMemberCacheService;
import com.hwrj.cloud.portal.service.UmsMemberService;
import com.hwrj.cloud.security.util.JwtTokenUtil;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * 会员管理Service实现类
 * Created by macro on 2018/8/3.
 */
@Slf4j
@Service
public class UmsMemberServiceImpl implements UmsMemberService {
    private static final Logger LOGGER = LoggerFactory.getLogger(com.hwrj.cloud.portal.service.impl.UmsMemberServiceImpl.class);
    @Autowired
    private PasswordEncoder passwordEncoder;
    @Autowired
    private JwtTokenUtil jwtTokenUtil;
    @Autowired
    private UmsMemberMapper memberMapper;
    @Autowired
    private UmsMemberLevelMapper memberLevelMapper;
    @Autowired
    private UmsMemberCacheService memberCacheService;

    @Autowired
    private UmsMemberPlatformMapper umsMemberPlatformMapper;

    @Autowired
    private UmsMemberDao umsMemberDao;

    @Value("${redis.key.authCode}")
    private String REDIS_KEY_PREFIX_AUTH_CODE;
    @Value("${redis.expire.authCode}")
    private Long AUTH_CODE_EXPIRE_SECONDS;

    @Override
    public UmsMember getByUsername(String username) {
        UmsMember member = memberCacheService.getMember(username);
        if (member != null) {
            return member;
        }
        UmsMemberExample example = new UmsMemberExample();
        example.createCriteria().andUsernameEqualTo(username);
        List<UmsMember> memberList = memberMapper.selectByExample(example);
        if (!CollectionUtils.isEmpty(memberList)) {
            member = memberList.get(0);
            memberCacheService.setMember(member);
            return member;
        }
        UmsMemberExample example1 = new UmsMemberExample();
        example.createCriteria().andUsernameEqualTo(username);
        List<UmsMember> memberList1 = memberMapper.selectByExample(example1);
        if (!CollectionUtils.isEmpty(memberList1)) {
            member = memberList1.get(0);
            memberCacheService.setMember(member);
            return member;
        }
        return null;
    }

    @Override
    public UmsMember getByPhone(String phone) {
        UmsMember member = null;
        UmsMemberExample example1 = new UmsMemberExample();
        example1.createCriteria().andUsernameEqualTo(phone);
        List<UmsMember> memberList1 = memberMapper.selectByExample(example1);
        if (!CollectionUtils.isEmpty(memberList1)) {
            member = memberList1.get(0);
            memberCacheService.setMember(member);
            return member;
        }
        return null;
    }

    @Override
    public UmsMember getById(Long id) {
        return memberMapper.selectByPrimaryKey(id);
    }

    @Override
    public void register(String username, String password, String telephone, String authCode) {
        //验证验证码
        if (!verifyAuthCode(authCode, telephone)) {
            Asserts.fail("验证码错误");
        }
        //查询是否已有该用户
        UmsMemberExample example = new UmsMemberExample();
        example.createCriteria().andUsernameEqualTo(username);
        example.or(example.createCriteria().andPhoneEqualTo(telephone));
        List<UmsMember> umsMembers = memberMapper.selectByExample(example);
        if (!CollectionUtils.isEmpty(umsMembers)) {
            Asserts.fail("该用户已经存在");
        }
        //没有该用户进行添加操作
        UmsMember umsMember = new UmsMember();
        umsMember.setUsername(username);
        umsMember.setPhone(telephone);
        umsMember.setPassword(passwordEncoder.encode(password));
        umsMember.setCreateTime(new Date());
        umsMember.setStatus(1);
        //获取默认会员等级并设置
        UmsMemberLevelExample levelExample = new UmsMemberLevelExample();
        levelExample.createCriteria().andDefaultStatusEqualTo(1);
        List<UmsMemberLevel> memberLevelList = memberLevelMapper.selectByExample(levelExample);
        if (!CollectionUtils.isEmpty(memberLevelList)) {
            umsMember.setMemberLevelId(memberLevelList.get(0).getId());
        }
        memberMapper.insert(umsMember);
        umsMember.setPassword(null);
    }

    @Override
    public String generateAuthCode(String telephone) {
        StringBuilder sb = new StringBuilder();
        Random random = new Random();
        for (int i = 0; i < 6; i++) {
            sb.append(random.nextInt(10));
        }
        memberCacheService.setAuthCode(telephone, sb.toString());
        return sb.toString();
    }

    @Override
    public String generateLoginCode(String telephone) {
        StringBuilder sb = new StringBuilder();
        Random random = new Random();
        for (int i = 0; i < 6; i++) {
            sb.append(random.nextInt(10));
        }
        memberCacheService.setLoginCode(telephone, sb.toString());
        return sb.toString();
    }

    @Override
    public void updatePassword(String telephone, String password, String authCode) {
        UmsMemberExample example = new UmsMemberExample();
        example.createCriteria().andPhoneEqualTo(telephone);
        List<UmsMember> memberList = memberMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(memberList)) {
            Asserts.fail("该账号不存在");
        }
        //验证验证码
        if (!verifyAuthCode(authCode, telephone)) {
            Asserts.fail("验证码错误");
        }
        UmsMember umsMember = memberList.get(0);
        umsMember.setPassword(passwordEncoder.encode(password));
        memberMapper.updateByPrimaryKeySelective(umsMember);
        memberCacheService.delMember(umsMember.getId());
    }

    @Override
    public UmsMember getCurrentMember() {
        SecurityContext ctx = SecurityContextHolder.getContext();
        Authentication auth = ctx.getAuthentication();
        MemberDetails memberDetails = (MemberDetails) auth.getPrincipal();
        return memberDetails.getUmsMember();
    }

    @Autowired
    private UmsCompanyShareMapper umsCompanyShareMapper;

    @Override
    public Map<String, Object> getCurrentWxUser() {
        SecurityContext ctx = SecurityContextHolder.getContext();
        Authentication auth = ctx.getAuthentication();
        MemberDetails memberDetails = (MemberDetails) auth.getPrincipal();
        UmsMember umsMember = memberDetails.getUmsMember();
        Map<String, Object> map = new HashMap<>();

        if (umsMember != null) {
            map.put("user", umsMember);
            UmsMemberPlatformExample example = new UmsMemberPlatformExample();
            UmsMemberPlatformExample.Criteria criteria = example.createCriteria();
            criteria.andMemberIdEqualTo(umsMember.getId());
            List<UmsMemberPlatform> platforms = umsMemberPlatformMapper.selectByExample(example);
            if (platforms == null || platforms.size() < 1) {

                throw new RuntimeException("登录失败");
            }
            UmsMemberPlatform umsMemberPlatform = platforms.get(0);
            map.put("wxUsers", umsMemberPlatform);
            Map<String, Object> ownData = umsMemberDao.getOwnData(umsMember.getId(), umsMemberPlatform.getId());
            map.put("pre", ownData);
            getCompany(umsMember, map);
        }
        return map;
    }

    @Override
    public UmsMemberPlatform getWxUser() {
        SecurityContext ctx = SecurityContextHolder.getContext();
        Authentication auth = ctx.getAuthentication();
        MemberDetails memberDetails = (MemberDetails) auth.getPrincipal();
        UmsMember umsMember = memberDetails.getUmsMember();
        Map<String, Object> map = new HashMap<>();
        UmsMemberPlatform umsMemberPlatform = null;
        if (umsMember != null) {
            map.put("user", umsMember);
            UmsMemberPlatformExample example = new UmsMemberPlatformExample();
            UmsMemberPlatformExample.Criteria criteria = example.createCriteria();
            criteria.andMemberIdEqualTo(umsMember.getId());
            List<UmsMemberPlatform> platforms = umsMemberPlatformMapper.selectByExample(example);
            if (platforms == null || platforms.size() < 1) {

                throw new RuntimeException("登录失败");
            }
            umsMemberPlatform = platforms.get(0);

        }
        return umsMemberPlatform;
    }

    @Autowired
    private UmsCompanyInfoMapper umsCompanyInfoMapper;

    private void getCompany(UmsMember umsMember, Map<String, Object> map) {
        UmsCompanyShareExample example = new UmsCompanyShareExample();
        UmsCompanyShareExample.Criteria criteria = example.createCriteria();
        criteria.andMemberIdEqualTo(umsMember.getId());
        criteria.andStatusEqualTo(0);
        List<UmsCompanyShare> umsCompanyShares = umsCompanyShareMapper.selectByExample(example);
        if (umsCompanyShares.size() > 0) {
            UmsCompanyShare umsCompanyShare = umsCompanyShares.get(0);

            Long compId = umsCompanyShare.getCompId();

            UmsCompanyInfo umsCompanyInfo = umsCompanyInfoMapper.selectByPrimaryKey(compId);
            if(umsCompanyInfo.getCompStatus().intValue()!=2){
                map.put("company",umsCompanyShare);
            }
        }
    }

    @Override
    public void updateIntegration(Long id, Integer integration) {
        UmsMember record = new UmsMember();
        record.setId(id);
        record.setIntegration(integration);
        memberMapper.updateByPrimaryKeySelective(record);
        memberCacheService.delMember(id);
    }

    @Override
    public UserDetails loadUserByUsername(String username) {
        UmsMember member = getByUsername(username);
        if (member != null) {
            return new MemberDetails(member);
        }
        throw new UsernameNotFoundException("用户名或密码错误");
    }


    @Override
    public String login(String username, String password) {
        String token = null;
        //密码需要客户端加密后传递
        try {
            UserDetails userDetails = loadUserByUsername(username);
            if (!passwordEncoder.matches(password, userDetails.getPassword())) {
                throw new BadCredentialsException("密码不正确");
            }
            UsernamePasswordAuthenticationToken authentication = new UsernamePasswordAuthenticationToken(userDetails, null, userDetails.getAuthorities());
            SecurityContextHolder.getContext().setAuthentication(authentication);
            token = jwtTokenUtil.generateToken(userDetails);
            memberCacheService.setToken(token, username);
        } catch (AuthenticationException e) {
            LOGGER.warn("登录异常:{}", e.getMessage());
        }
        return token;
    }

    @Override
    public String loginPhone(String phone, String code) {
        if (StringUtils.isEmpty(code)) {
            throw new RuntimeException("验证码错误");
        }
        String token = null;
        //密码需要客户端加密后传递
        try {
            UserDetails userDetails = new MemberDetails(getByPhone(phone));
            String loginCode = memberCacheService.getLoginCode(phone);
            if (!code.equals(loginCode)) {
                throw new BadCredentialsException("验证码不正确");
            }
            UsernamePasswordAuthenticationToken authentication = new UsernamePasswordAuthenticationToken(userDetails, null, userDetails.getAuthorities());
            SecurityContextHolder.getContext().setAuthentication(authentication);
            token = jwtTokenUtil.generateToken(userDetails);
            memberCacheService.setToken(token, userDetails.getUsername());
        } catch (AuthenticationException e) {
            LOGGER.warn("登录异常:{}", e.getMessage());
        }
        return token;
    }

    @Override
    public String refreshToken(String token) {
        return jwtTokenUtil.refreshHeadToken(token);
    }

    @Override
    public Long getMemberId() {
        UmsMember member = getCurrentMember();
        return member.getId();
    }

    //对输入的验证码进行校验
    private boolean verifyAuthCode(String authCode, String telephone) {
        if (StringUtils.isEmpty(authCode)) {
            return false;
        }
        String realAuthCode = memberCacheService.getAuthCode(telephone);
        return authCode.equals(realAuthCode);
    }

}
